<div class="row"><div class="col-sm-6 col-sm-offset-3">
    <panel class="notification-box">
        <panel-heading>Loading...</panel-heading>
        <panel-body class="text-center">
            <i class="fa fa-spinner fa-spin fa-2x"></i>
        </panel-body>
    </panel>
</div></div>
